==> execution-impl-202.6397.94-sources.jar: com/intellij/diagnostic/logging/LogsFragment.java <==
// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.diagnostic.logging;

import com.intellij.diagnostic.DiagnosticBundle;
import com.intellij.execution.ExecutionBundle;
import com.intellij.execution.configurations.LogFileOptions;
import com.intellij.execution.configurations.PredefinedLogFile;
import com.intellij.execution.configurations.RunConfigurationBase;
import com.intellij.execution.ui.NestedGroupFragment;
import com.intellij.execution.ui.SettingsEditorFragment;
import com.intellij.openapi.actionSystem.ActionToolbarPosition;
import com.intellij.openapi.fileChooser.FileChooserDescriptorFactory;
import com.intellij.openapi.options.ConfigurationException;
import com.intellij.openapi.ui.LabeledComponent;
import com.intellij.openapi.ui.TextComponentAccessor;
import com.intellij.openapi.ui.TextFieldWithBrowseButton;
import com.intellij.openapi.util.text.StringUtil;
import com.intellij.openapi.wm.IdeFocusManager;
import com.intellij.ui.BooleanTableCellRenderer;
import com.intellij.ui.TableUtil;
import com.intellij.ui.ToolbarDecorator;
import com.intellij.ui.table.TableView;
import com.intellij.util.SmartList;
import com.intellij.util.ui.ColumnInfo;
import com.intellij.util.ui.ListTableModel;
import com.intellij.util.ui.LocalPathCellEditor;
import gnu.trove.THashMap;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.table.*;
import java.awt.*;
import java.util.List;
import java.util.*;

public class LogsFragment<T extends RunConfigurationBase<?>> extends NestedGroupFragment<T> {

  private final Map<LogFileOptions, PredefinedLogFile> myLog2Predefined = new THashMap<>();
  private final List<PredefinedLogFile> myUnresolvedPredefined = new SmartList<>();
  private final TableView<LogFileOptions> myFilesTable;
  private final ListTableModel<LogFileOptions> myModel;

  public LogsFragment() {
    super("log.monitor", DiagnosticBundle.message("log.monitor.fragment.name"), DiagnosticBundle.message("log.monitor.fragment.group"), t -> !t.getLogFiles().isEmpty());

    ColumnInfo<LogFileOptions, String> TAB_NAME = new TabNameColumnInfo();
    ColumnInfo<LogFileOptions, String> FILE = new FileColumnInfo();
    ColumnInfo<LogFileOptions, Boolean> IS_SHOW = new MyIsActiveColumnInfo();
    ColumnInfo<LogFileOptions, Boolean> IS_SKIP_CONTENT = new MyIsSkipColumnInfo();

    myModel = new ListTableModel<>(TAB_NAME, FILE, IS_SHOW, IS_SKIP_CONTENT);
    myFilesTable = new TableView<>(myModel);
    myFilesTable.getEmptyText().setText(DiagnosticBundle.message("log.monitor.no.files"));

    final JTableHeader tableHeader = myFilesTable.getTableHeader();
    final FontMetrics fontMetrics = tableHeader.getFontMetrics(tableHeader.getFont());

    int preferredWidth = fontMetrics.stringWidth(IS_SHOW.getName()) + 20;
    setUpColumnWidth(tableHeader, preferredWidth, 2);

    preferredWidth = fontMetrics.stringWidth(IS_SKIP_CONTENT.getName()) + 20;
    setUpColumnWidth(tableHeader, preferredWidth, 3);

    setUpColumnWidth(tableHeader, 100, 0);

    myFilesTable.setColumnSelectionAllowed(false);
    myFilesTable.setShowGrid(false);
    myFilesTable.setDragEnabled(false);
    myFilesTable.setShowHorizontalLines(false);
    myFilesTable.setShowVerticalLines(false);
    myFilesTable.setIntercellSpacing(new Dimension(0, 0));
    myFilesTable.setupEasyFocusTraversing();

    myComponent = ToolbarDecorator.createDecorator(myFilesTable)
      .setToolbarPosition(ActionToolbarPosition.BOTTOM)
      .setAddAction(button -> {
        ArrayList<LogFileOptions> newList = new ArrayList<>(myModel.getItems());
        LogFileOptions newOptions = new LogFileOptions("", "", true);
        if (showEditorDialog(newOptions)) {
          newList.add(newOptions);
          myModel.setItems(newList);
          int index = myModel.getRowCount() - 1;
          myModel.fireTableRowsInserted(index, index);
          myFilesTable.setRowSelectionInterval(index, index);
        }
      }).setRemoveAction(button -> {
        TableUtil.stopEditing(myFilesTable);
        final int[] selected = myFilesTable.getSelectedRows();
        if (selected.length == 0) return;
        for (int i = selected.length - 1; i >= 0; i--) {
          myModel.removeRow(selected[i]);
        }
        for (int i = selected.length - 1; i >= 0; i--) {
          int idx = selected[i];
          myModel.fireTableRowsDeleted(idx, idx);
        }
        int selection = selected[0];
        if (selection >= myModel.getRowCount()) {
          selection = myModel.getRowCount() - 1;
        }
        if (selection >= 0) {
          myFilesTable.setRowSelectionInterval(selection, selection);
        }
        IdeFocusManager.getGlobalInstance()
          .doWhenFocusSettlesDown(() -> IdeFocusManager.getGlobalInstance().requestFocus(myFilesTable, true));
      }).setEditAction(button -> {
        final int selectedRow = myFilesTable.getSelectedRow();
        //noinspection ConstantConditions
        showEditorDialog(myFilesTable.getSelectedObject());
        myModel.fireTableDataChanged();
        myFilesTable.setRowSelectionInterval(selectedRow, selectedRow);
      }).setRemoveActionUpdater(e -> myFilesTable.getSelectedRowCount() >= 1 &&
                                     !myLog2Predefined.containsKey(myFilesTable.getSelectedObject())).setEditActionUpdater(
        e -> myFilesTable.getSelectedRowCount() >= 1 &&
             !myLog2Predefined.containsKey(myFilesTable.getSelectedObject()) &&
             myFilesTable.getSelectedObject() != null).disableUpDownActions().createPanel();
  }

  private void setUpColumnWidth(final JTableHeader tableHeader, final int preferredWidth, int columnIdx) {
    myFilesTable.getColumnModel().getColumn(columnIdx).setCellRenderer(new BooleanTableCellRenderer());
    final TableColumn tableColumn = tableHeader.getColumnModel().getColumn(columnIdx);
    tableColumn.setWidth(preferredWidth);
    tableColumn.setPreferredWidth(preferredWidth);
    tableColumn.setMinWidth(preferredWidth);
    tableColumn.setMaxWidth(preferredWidth);
  }

  @Override
  protected void resetEditorFrom(@NotNull T configuration) {
    super.resetEditorFrom(configuration);
    ArrayList<LogFileOptions> list = new ArrayList<>();
    final List<LogFileOptions> logFiles = configuration.getLogFiles();
    for (LogFileOptions setting : logFiles) {
      list.add(
        new LogFileOptions(setting.getName(), setting.getPathPattern(), setting.isEnabled(), setting.isSkipContent(), setting.isShowAll()));
    }
    myLog2Predefined.clear();
    myUnresolvedPredefined.clear();
    final List<PredefinedLogFile> predefinedLogFiles = configuration.getPredefinedLogFiles();
    for (PredefinedLogFile predefinedLogFile : predefinedLogFiles) {
      PredefinedLogFile logFile = new PredefinedLogFile();
      logFile.copyFrom(predefinedLogFile);
      final LogFileOptions options = configuration.getOptionsForPredefinedLogFile(logFile);
      if (options != null) {
        myLog2Predefined.put(options, logFile);
        list.add(options);
      }
      else {
        myUnresolvedPredefined.add(logFile);
      }
    }
    myModel.setItems(list);
  }

  @Override
  protected void applyEditorTo(@NotNull T configuration) throws ConfigurationException {
    super.applyEditorTo(configuration);
    configuration.removeAllLogFiles();
    configuration.removeAllPredefinedLogFiles();
    if (!isSelected()) return;
    for (int i = 0; i < myModel.getRowCount(); i++) {
      LogFileOptions options = myModel.getItem(i);
      if (Objects.equals(options.getPathPattern(), "")) {
        continue;
      }
      final Boolean checked = (Boolean)myModel.getValueAt(i, 2);
      final Boolean skipped = (Boolean)myModel.getValueAt(i, 3);
      final PredefinedLogFile predefined = myLog2Predefined.get(options);
      if (predefined != null) {
        PredefinedLogFile file = new PredefinedLogFile();
        file.setId(predefined.getId());
        file.setEnabled(options.isEnabled());
        configuration.addPredefinedLogFile(file);
      }
      else {
        configuration
          .addLogFile(options.getPathPattern(), options.getName(), checked.booleanValue(), skipped.booleanValue(), options.isShowAll());
      }
    }
    for (PredefinedLogFile logFile : myUnresolvedPredefined) {
      configuration.addPredefinedLogFile(logFile);
    }
  }

  @Override
  protected List<SettingsEditorFragment<T, ?>> createChildren() {
    TextFieldWithBrowseButton myOutputFile = new TextFieldWithBrowseButton();
    myOutputFile.addBrowseFolderListener(ExecutionBundle.message("choose.file.to.save.console.output"),
                                         ExecutionBundle.message("console.output.would.be.saved.to.the.specified.file"), null,
                                         FileChooserDescriptorFactory.createSingleFileOrFolderDescriptor(),
                                         TextComponentAccessor.TEXT_FIELD_WHOLE_TEXT);
    LabeledComponent<TextFieldWithBrowseButton> component =
      LabeledComponent.create(myOutputFile, ExecutionBundle.message("save.output.console.to.file"), BorderLayout.WEST);
    SettingsEditorFragment<T, LabeledComponent<TextFieldWithBrowseButton>> fragment =
      new SettingsEditorFragment<>("logs.save.output", ExecutionBundle.message("save.output.console.to.file.option"), null, component,
                                   (t, component1) -> component1.getComponent().setText(
                                     StringUtil.notNullize(t.getOutputFilePath())),
                                   (t, component1) -> {
                                     t.setFileOutputPath(component1.getComponent().getText());
                                     t.setSaveOutputToFile(StringUtil.isNotEmpty(component.getComponent().getText()));
                                   },
                                   t -> t.isSaveOutputToFile());
    SettingsEditorFragment<T, ?> stdOut = SettingsEditorFragment
      .createTag("logs.stdout", DiagnosticBundle.message("log.monitor.fragment.stdout"), null, t -> t.isShowConsoleOnStdOut(),
                 (t, value) -> t.setShowConsoleOnStdOut(value));
    SettingsEditorFragment<T, ?> stdErr = SettingsEditorFragment
      .createTag("logs.stderr", DiagnosticBundle.message("log.monitor.fragment.stderr"), null, t -> t.isShowConsoleOnStdErr(),
                 (t, value) -> t.setShowConsoleOnStdErr(value));
    return Arrays.asList(fragment, stdOut, stdErr);
  }

  @Override
  public @Nullable String getChildrenGroupName() {
    return DiagnosticBundle.message("log.monitor.fragment.settings");
  }

  private class TabNameColumnInfo extends ColumnInfo<LogFileOptions, String> {
    TabNameColumnInfo() {
      super(DiagnosticBundle.message("log.monitor.tab.name.column"));
    }

    @Override
    public @Nullable TableCellRenderer getRenderer(LogFileOptions options) {
      return new DefaultTableCellRenderer();
    }

    @Override
    public String valueOf(final LogFileOptions object) {
      return object.getName();
    }

    @Override
    public void setValue(LogFileOptions options, String value) {
      options.setName(value);
    }

    @Override
    public boolean isCellEditable(final LogFileOptions o) {
      return !myLog2Predefined.containsKey(o);
    }
  }

  private class FileColumnInfo extends ColumnInfo<LogFileOptions, String> {
    FileColumnInfo() {
      super(DiagnosticBundle.message("log.monitor.file.column"));
    }

    @Override
    public String valueOf(final LogFileOptions object) {
      return object.getPathPattern();
    }

    @Override
    public void setValue(LogFileOptions options, String value) {
      options.setPathPattern(value);
    }

    @Override
    public boolean isCellEditable(final LogFileOptions o) {
      return !myLog2Predefined.containsKey(o);
    }

    @Override
    public @Nullable TableCellEditor getEditor(LogFileOptions options) {
      return new LocalPathCellEditor();
    }
  }

  private class MyIsActiveColumnInfo extends ColumnInfo<LogFileOptions, Boolean> {
    protected MyIsActiveColumnInfo() {
      super(DiagnosticBundle.message("log.monitor.is.active.column"));
    }

    @Override
    public Class<?> getColumnClass() {
      return Boolean.class;
    }

    @Override
    public Boolean valueOf(final LogFileOptions object) {
      return object.isEnabled();
    }

    @Override
    public boolean isCellEditable(LogFileOptions element) {
      return true;
    }

    @Override
    public void setValue(LogFileOptions element, Boolean checked) {
      final PredefinedLogFile predefinedLogFile = myLog2Predefined.get(element);
      if (predefinedLogFile != null) {
        predefinedLogFile.setEnabled(checked.booleanValue());
      }
      element.setEnabled(checked.booleanValue());
    }
  }

  private class MyIsSkipColumnInfo extends ColumnInfo<LogFileOptions, Boolean> {
    protected MyIsSkipColumnInfo() {
      super(DiagnosticBundle.message("log.monitor.is.skipped.column"));
    }

    @Override
    public Class<?> getColumnClass() {
      return Boolean.class;
    }

    @Override
    public Boolean valueOf(final LogFileOptions element) {
      return element.isSkipContent();
    }

    @Override
    public boolean isCellEditable(LogFileOptions element) {
      return !myLog2Predefined.containsKey(element);
    }

    @Override
    public void setValue(LogFileOptions element, Boolean skipped) {
      element.setSkipContent(skipped.booleanValue());
    }
  }

  private static boolean showEditorDialog(@NotNull LogFileOptions options) {
    EditLogPatternDialog dialog = new EditLogPatternDialog();
    dialog.init(options.getName(), options.getPathPattern(), options.isShowAll());
    if (dialog.showAndGet()) {
      options.setName(dialog.getName());
      options.setPathPattern(dialog.getLogPattern());
      options.setShowAll(dialog.isShowAllFiles());
      return true;
    }
    return false;
  }
}

==> execution-impl-203.5981.155-sources.jar: com/intellij/diagnostic/logging/LogsFragment.java <==
// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.diagnostic.logging;

import com.intellij.diagnostic.DiagnosticBundle;
import com.intellij.execution.ExecutionBundle;
import com.intellij.execution.configurations.LogFileOptions;
import com.intellij.execution.configurations.PredefinedLogFile;
import com.intellij.execution.configurations.RunConfigurationBase;
import com.intellij.execution.ui.NestedGroupFragment;
import com.intellij.execution.ui.SettingsEditorFragment;
import com.intellij.openapi.actionSystem.ActionToolbarPosition;
import com.intellij.openapi.fileChooser.FileChooserDescriptorFactory;
import com.intellij.openapi.ui.LabeledComponent;
import com.intellij.openapi.ui.TextComponentAccessor;
import com.intellij.openapi.ui.TextFieldWithBrowseButton;
import com.intellij.openapi.util.text.StringUtil;
import com.intellij.openapi.wm.IdeFocusManager;
import com.intellij.ui.BooleanTableCellRenderer;
import com.intellij.ui.TableUtil;
import com.intellij.ui.ToolbarDecorator;
import com.intellij.ui.table.TableView;
import com.intellij.util.SmartList;
import com.intellij.util.ui.ColumnInfo;
import com.intellij.util.ui.ListTableModel;
import com.intellij.util.ui.LocalPathCellEditor;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.table.*;
import java.awt.*;
import java.util.List;
import java.util.*;

public final class LogsFragment<T extends RunConfigurationBase<?>> extends NestedGroupFragment<T> {
  private final Map<LogFileOptions, PredefinedLogFile> myLog2Predefined = new HashMap<>();
  private final List<PredefinedLogFile> myUnresolvedPredefined = new SmartList<>();
  private final TableView<LogFileOptions> myFilesTable;
  private final ListTableModel<LogFileOptions> myModel;

  public LogsFragment() {
    super("log.monitor",
          DiagnosticBundle.message("log.monitor.fragment.name"), DiagnosticBundle.message("log.monitor.fragment.group"),
          t -> !t.getLogFiles().isEmpty());
    setActionHint(ExecutionBundle.message("the.ide.will.display.the.selected.logs.in.the.run.tool.window"));

    ColumnInfo<LogFileOptions, String> TAB_NAME = new TabNameColumnInfo();
    ColumnInfo<LogFileOptions, String> FILE = new FileColumnInfo();
    ColumnInfo<LogFileOptions, Boolean> IS_SHOW = new MyIsActiveColumnInfo();
    ColumnInfo<LogFileOptions, Boolean> IS_SKIP_CONTENT = new MyIsSkipColumnInfo();

    myModel = new ListTableModel<>(TAB_NAME, FILE, IS_SHOW, IS_SKIP_CONTENT);
    myFilesTable = new TableView<>(myModel);
    myFilesTable.getEmptyText().setText(DiagnosticBundle.message("log.monitor.no.files"));

    final JTableHeader tableHeader = myFilesTable.getTableHeader();
    final FontMetrics fontMetrics = tableHeader.getFontMetrics(tableHeader.getFont());

    int preferredWidth = fontMetrics.stringWidth(IS_SHOW.getName()) + 20;
    setUpColumnWidth(tableHeader, preferredWidth, 2);

    preferredWidth = fontMetrics.stringWidth(IS_SKIP_CONTENT.getName()) + 20;
    setUpColumnWidth(tableHeader, preferredWidth, 3);

    setUpColumnWidth(tableHeader, 100, 0);

    myFilesTable.setColumnSelectionAllowed(false);
    myFilesTable.setShowGrid(false);
    myFilesTable.setDragEnabled(false);
    myFilesTable.setShowHorizontalLines(false);
    myFilesTable.setShowVerticalLines(false);
    myFilesTable.setIntercellSpacing(new Dimension(0, 0));
    myFilesTable.setupEasyFocusTraversing();

    myComponent = ToolbarDecorator.createDecorator(myFilesTable)
      .setToolbarPosition(ActionToolbarPosition.BOTTOM)
      .setAddAction(button -> {
        List<LogFileOptions> newList = new ArrayList<>(myModel.getItems());
        LogFileOptions newOptions = new LogFileOptions("", "", true);
        if (showEditorDialog(newOptions)) {
          newList.add(newOptions);
          myModel.setItems(newList);
          int index = myModel.getRowCount() - 1;
          myModel.fireTableRowsInserted(index, index);
          myFilesTable.setRowSelectionInterval(index, index);
        }
      }).setRemoveAction(button -> {
        TableUtil.stopEditing(myFilesTable);
        final int[] selected = myFilesTable.getSelectedRows();
        if (selected.length == 0) return;
        for (int i = selected.length - 1; i >= 0; i--) {
          myModel.removeRow(selected[i]);
        }
        for (int i = selected.length - 1; i >= 0; i--) {
          int idx = selected[i];
          myModel.fireTableRowsDeleted(idx, idx);
        }
        int selection = selected[0];
        if (selection >= myModel.getRowCount()) {
          selection = myModel.getRowCount() - 1;
        }
        if (selection >= 0) {
          myFilesTable.setRowSelectionInterval(selection, selection);
        }
        IdeFocusManager.getGlobalInstance()
          .doWhenFocusSettlesDown(() -> IdeFocusManager.getGlobalInstance().requestFocus(myFilesTable, true));
      }).setEditAction(button -> {
        final int selectedRow = myFilesTable.getSelectedRow();
        //noinspection ConstantConditions
        showEditorDialog(myFilesTable.getSelectedObject());
        myModel.fireTableDataChanged();
        myFilesTable.setRowSelectionInterval(selectedRow, selectedRow);
      }).setRemoveActionUpdater(e -> myFilesTable.getSelectedRowCount() >= 1 &&
                                     !myLog2Predefined.containsKey(myFilesTable.getSelectedObject())).setEditActionUpdater(
        e -> myFilesTable.getSelectedRowCount() >= 1 &&
             !myLog2Predefined.containsKey(myFilesTable.getSelectedObject()) &&
             myFilesTable.getSelectedObject() != null).disableUpDownActions().createPanel();
  }

  private void setUpColumnWidth(final JTableHeader tableHeader, final int preferredWidth, int columnIdx) {
    myFilesTable.getColumnModel().getColumn(columnIdx).setCellRenderer(new BooleanTableCellRenderer());
    final TableColumn tableColumn = tableHeader.getColumnModel().getColumn(columnIdx);
    tableColumn.setWidth(preferredWidth);
    tableColumn.setPreferredWidth(preferredWidth);
    tableColumn.setMinWidth(preferredWidth);
    tableColumn.setMaxWidth(preferredWidth);
  }

  @Override
  protected void resetEditorFrom(@NotNull T configuration) {
    super.resetEditorFrom(configuration);
    List<LogFileOptions> list = new ArrayList<>();
    final List<LogFileOptions> logFiles = configuration.getLogFiles();
    for (LogFileOptions setting : logFiles) {
      list.add(
        new LogFileOptions(setting.getName(), setting.getPathPattern(), setting.isEnabled(), setting.isSkipContent(), setting.isShowAll()));
    }
    myLog2Predefined.clear();
    myUnresolvedPredefined.clear();
    final List<PredefinedLogFile> predefinedLogFiles = configuration.getPredefinedLogFiles();
    for (PredefinedLogFile predefinedLogFile : predefinedLogFiles) {
      PredefinedLogFile logFile = new PredefinedLogFile();
      logFile.copyFrom(predefinedLogFile);
      final LogFileOptions options = configuration.getOptionsForPredefinedLogFile(logFile);
      if (options != null) {
        myLog2Predefined.put(options, logFile);
        list.add(options);
      }
      else {
        myUnresolvedPredefined.add(logFile);
      }
    }
    myModel.setItems(list);
  }

  @Override
  protected void applyEditorTo(@NotNull T configuration) {
    super.applyEditorTo(configuration);
    configuration.removeAllLogFiles();
    configuration.removeAllPredefinedLogFiles();
    if (!isSelected()) return;
    for (int i = 0; i < myModel.getRowCount(); i++) {
      LogFileOptions options = myModel.getItem(i);
      if (Objects.equals(options.getPathPattern(), "")) {
        continue;
      }
      final Boolean checked = (Boolean)myModel.getValueAt(i, 2);
      final Boolean skipped = (Boolean)myModel.getValueAt(i, 3);
      final PredefinedLogFile predefined = myLog2Predefined.get(options);
      if (predefined != null) {
        PredefinedLogFile file = new PredefinedLogFile();
        file.setId(predefined.getId());
        file.setEnabled(options.isEnabled());
        configuration.addPredefinedLogFile(file);
      }
      else {
        configuration
          .addLogFile(options.getPathPattern(), options.getName(), checked.booleanValue(), skipped.booleanValue(), options.isShowAll());
      }
    }
    for (PredefinedLogFile logFile : myUnresolvedPredefined) {
      configuration.addPredefinedLogFile(logFile);
    }
  }

  @Override
  protected List<SettingsEditorFragment<T, ?>> createChildren() {
    TextFieldWithBrowseButton myOutputFile = new TextFieldWithBrowseButton();
    myOutputFile.addBrowseFolderListener(ExecutionBundle.message("choose.file.to.save.console.output"),
                                         ExecutionBundle.message("console.output.would.be.saved.to.the.specified.file"), null,
                                         FileChooserDescriptorFactory.createSingleFileOrFolderDescriptor(),
                                         TextComponentAccessor.TEXT_FIELD_WHOLE_TEXT);
    LabeledComponent<TextFieldWithBrowseButton> component =
      LabeledComponent.create(myOutputFile, ExecutionBundle.message("save.output.console.to.file"), BorderLayout.WEST);
    SettingsEditorFragment<T, LabeledComponent<TextFieldWithBrowseButton>> fragment =
      new SettingsEditorFragment<>("logs.save.output", ExecutionBundle.message("save.output.console.to.file.option"), null, component,
                                   (t, component1) -> component1.getComponent().setText(
                                     StringUtil.notNullize(t.getOutputFilePath())),
                                   (t, component1) -> {
                                     t.setFileOutputPath(component1.getComponent().getText());
                                     t.setSaveOutputToFile(StringUtil.isNotEmpty(component.getComponent().getText()));
                                   },
                                   t -> t.isSaveOutputToFile());
    fragment.setActionHint(ExecutionBundle.message("write.the.output.of.the.application.to.a.file.for.later.inspection"));
    SettingsEditorFragment<T, ?> stdOut = SettingsEditorFragment
      .createTag("logs.stdout", DiagnosticBundle.message("log.monitor.fragment.stdout"), null, t -> t.isShowConsoleOnStdOut(),
                 (t, value) -> t.setShowConsoleOnStdOut(value));
    stdOut.setActionHint(ExecutionBundle.message("activate.the.console.when.the.application.writes.to.the.standard.output.stream"));
    SettingsEditorFragment<T, ?> stdErr = SettingsEditorFragment
      .createTag("logs.stderr", DiagnosticBundle.message("log.monitor.fragment.stderr"), null, t -> t.isShowConsoleOnStdErr(),
                 (t, value) -> t.setShowConsoleOnStdErr(value));
    stdErr.setActionHint(ExecutionBundle.message("activate.the.console.when.the.application.writes.to.the.standard.error.stream"));
    return Arrays.asList(fragment, stdOut, stdErr);
  }

  @Override
  public String getChildrenGroupName() {
    return DiagnosticBundle.message("log.monitor.fragment.settings");
  }

  private class TabNameColumnInfo extends ColumnInfo<LogFileOptions, String> {
    TabNameColumnInfo() {
      super(DiagnosticBundle.message("log.monitor.tab.name.column"));
    }

    @Override
    public @Nullable TableCellRenderer getRenderer(LogFileOptions options) {
      return new DefaultTableCellRenderer();
    }

    @Override
    public String valueOf(final LogFileOptions object) {
      return object.getName();
    }

    @Override
    public void setValue(LogFileOptions options, String value) {
      options.setName(value);
    }

    @Override
    public boolean isCellEditable(final LogFileOptions o) {
      return !myLog2Predefined.containsKey(o);
    }
  }

  private class FileColumnInfo extends ColumnInfo<LogFileOptions, String> {
    FileColumnInfo() {
      super(DiagnosticBundle.message("log.monitor.file.column"));
    }

    @Override
    public String valueOf(final LogFileOptions object) {
      return object.getPathPattern();
    }

    @Override
    public void setValue(LogFileOptions options, String value) {
      options.setPathPattern(value);
    }

    @Override
    public boolean isCellEditable(final LogFileOptions o) {
      return !myLog2Predefined.containsKey(o);
    }

    @Override
    public @Nullable TableCellEditor getEditor(LogFileOptions options) {
      return new LocalPathCellEditor();
    }
  }

  private class MyIsActiveColumnInfo extends ColumnInfo<LogFileOptions, Boolean> {
    protected MyIsActiveColumnInfo() {
      super(DiagnosticBundle.message("log.monitor.is.active.column"));
    }

    @Override
    public Class<?> getColumnClass() {
      return Boolean.class;
    }

    @Override
    public Boolean valueOf(final LogFileOptions object) {
      return object.isEnabled();
    }

    @Override
    public boolean isCellEditable(LogFileOptions element) {
      return true;
    }

    @Override
    public void setValue(LogFileOptions element, Boolean checked) {
      final PredefinedLogFile predefinedLogFile = myLog2Predefined.get(element);
      if (predefinedLogFile != null) {
        predefinedLogFile.setEnabled(checked.booleanValue());
      }
      element.setEnabled(checked.booleanValue());
    }
  }

  private class MyIsSkipColumnInfo extends ColumnInfo<LogFileOptions, Boolean> {
    protected MyIsSkipColumnInfo() {
      super(DiagnosticBundle.message("log.monitor.is.skipped.column"));
    }

    @Override
    public Class<?> getColumnClass() {
      return Boolean.class;
    }

    @Override
    public Boolean valueOf(final LogFileOptions element) {
      return element.isSkipContent();
    }

    @Override
    public boolean isCellEditable(LogFileOptions element) {
      return !myLog2Predefined.containsKey(element);
    }

    @Override
    public void setValue(LogFileOptions element, Boolean skipped) {
      element.setSkipContent(skipped.booleanValue());
    }
  }

  private static boolean showEditorDialog(@NotNull LogFileOptions options) {
    EditLogPatternDialog dialog = new EditLogPatternDialog();
    dialog.init(options.getName(), options.getPathPattern(), options.isShowAll());
    if (dialog.showAndGet()) {
      options.setName(dialog.getName());
      options.setPathPattern(dialog.getLogPattern());
      options.setShowAll(dialog.isShowAllFiles());
      return true;
    }
    return false;
  }
}
